﻿/// \file FrmMain.cs
/// \brief Formulario para configurar los parametros
/// \version 1.0
/// \author SUNAT
/// \date 28 de Julio de 2014
#region using directives
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using System.Configuration;
#endregion

namespace WinClienteSEIDA
{
    public partial class FrmConfigure : Form
    {
        private bool m_change = false;

        public FrmConfigure()
        {
            InitializeComponent();
        }

        #region Events
        private void FrmConfigure_Load(object sender, EventArgs e)
        {
            Configuration config = ConfigurationManager.OpenExeConfiguration(Application.ExecutablePath);
            this.txtRUC.Text = config.AppSettings.Settings["NumeroRUC"].Value;
            this.txtUsuario.Text = config.AppSettings.Settings["UsuarioSOL"].Value;
            this.txtCodigoEnvio.Text = config.AppSettings.Settings["CodigoEnvio"].Value;
            this.txtUrl.Text = config.AppSettings.Settings["UrlServicioWeb"].Value;
            this.txtDir1.Text = config.AppSettings.Settings["DirectorioDocumento"].Value;
            this.txtDir2.Text = config.AppSettings.Settings["DirectorioAcuse"].Value;
            this.txtDir3.Text = config.AppSettings.Settings["DirectorioRespuesta"].Value;

            if (config.AppSettings.Settings["ProxyActivo"].Value.ToLower() == Logic.Core.CONSTANTS_TRUE)
            {
                this.chkActivarProxy.Checked = true;
            }
            else
            {
                this.chkActivarProxy.Checked = false;
            }
            this.txtDireccionProxy.Text = config.AppSettings.Settings["ProxyDireccionIP"].Value;
            this.txtPuertoProxy.Text = config.AppSettings.Settings["ProxyPuerto"].Value;
            this.txtDominioProxy.Text = config.AppSettings.Settings["ProxyNombreDominio"].Value;
            this.txtUsuarioProxy.Text = config.AppSettings.Settings["ProxyUsuario"].Value;
            this.txtClaveProxy.Text = config.AppSettings.Settings["ProxyClave"].Value;
        }

        private void cmdGrabar_Click(object sender, EventArgs e)
        {
            bool passed = false;
            long intParsed=0;
            if (long.TryParse(this.txtRUC.Text.Trim(), out intParsed))
            {
                passed = true;
            }
            else
            {
                MessageBox.Show("Error, el número de RUC es incorrecto!", "Aviso");
            }
            if (this.txtPuertoProxy.Text.Trim().Length>0)
            {
                if (long.TryParse(this.txtPuertoProxy.Text.Trim(), out intParsed))
                {
                    passed = true;
                }
                else
                {
                    passed = false;
                    MessageBox.Show("Error, el número de puerto del servidor proxy es incorrecto!", "Aviso");
                }
            }

            if (passed)
            {
                Configuration config = ConfigurationManager.OpenExeConfiguration(Application.ExecutablePath);
                config.AppSettings.Settings.Remove("NumeroRUC");
                config.AppSettings.Settings.Add("NumeroRUC", this.txtRUC.Text);
                config.AppSettings.Settings.Remove("UsuarioSOL");
                config.AppSettings.Settings.Add("UsuarioSOL", this.txtUsuario.Text);
                config.AppSettings.Settings.Remove("CodigoEnvio");
                config.AppSettings.Settings.Add("CodigoEnvio", this.txtCodigoEnvio.Text);
                config.AppSettings.Settings.Remove("UrlServicioWeb");
                config.AppSettings.Settings.Add("UrlServicioWeb", this.txtUrl.Text);
                config.AppSettings.Settings.Remove("DirectorioDocumento");
                config.AppSettings.Settings.Add("DirectorioDocumento", this.txtDir1.Text);
                config.AppSettings.Settings.Remove("DirectorioAcuse");
                config.AppSettings.Settings.Add("DirectorioAcuse", this.txtDir2.Text);
                config.AppSettings.Settings.Remove("DirectorioRespuesta");
                config.AppSettings.Settings.Add("DirectorioRespuesta", this.txtDir3.Text);

                if (this.chkActivarProxy.Checked)
                {
                    config.AppSettings.Settings.Remove("ProxyActivo");
                    config.AppSettings.Settings.Add("ProxyActivo", Logic.Core.CONSTANTS_TRUE);
                }
                else
                {
                    config.AppSettings.Settings.Remove("ProxyActivo");
                    config.AppSettings.Settings.Add("ProxyActivo", Logic.Core.CONSTANTS_FALSE);
                }
                config.AppSettings.Settings.Remove("ProxyDireccionIP");
                config.AppSettings.Settings.Add("ProxyDireccionIP", this.txtDireccionProxy.Text);
                config.AppSettings.Settings.Remove("ProxyPuerto");
                config.AppSettings.Settings.Add("ProxyPuerto", this.txtPuertoProxy.Text);
                config.AppSettings.Settings.Remove("ProxyNombreDominio");
                config.AppSettings.Settings.Add("ProxyNombreDominio", this.txtDominioProxy.Text);
                config.AppSettings.Settings.Remove("ProxyUsuario");
                config.AppSettings.Settings.Add("ProxyUsuario", this.txtUsuarioProxy.Text);
                config.AppSettings.Settings.Remove("ProxyClave");
                config.AppSettings.Settings.Add("ProxyClave", this.txtClaveProxy.Text);

                config.Save(ConfigurationSaveMode.Full);
                m_change = true;
                this.Close();
            }
        }

        private void cmdCerrar_Click(object sender, EventArgs e)
        {
            m_change = false;
            this.Close();
        }

        public bool Change
        {
            get { return m_change; }
            set { m_change = value; }
        }

        private void radioButton1_CheckedChanged(object sender, EventArgs e)
        {
            txtUrl.Text = Logic.Core.URL_PRODUCCION;
        }

        private void radioButton2_CheckedChanged(object sender, EventArgs e)
        {
            txtUrl.Text = Logic.Core.URL_PRUEBAS;
        }
        #endregion

    }
}

