﻿/// \file FrmMain.cs
/// \brief Formulario principal para enviar y/o consultar al servicio de SEIDA
/// \version 1.0
/// \author SUNAT
/// \date 28 de Julio de 2014
#region using directives
using System;
using System.Diagnostics;
using System.Net;
using System.IO;
using System.Collections.Generic;
using System.Collections;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using System.Configuration;
using System.Threading;
#endregion

namespace WinClienteSEIDA
{
    public partial class FrmMain : Form
    {
        #region Privates Variables
        private DataTable m_dtEnvios = null; ///< Objeto DataTable para manipular los registros a enviar
        private DataTable m_dtConsultas = null;  ///< Objeto DataTable para manipular los registros a recibir
        private string m_url; ///< Direccion URL del servicio Web
        private string m_pathApp; ///< Ruta de ejecucion de la aplicacion
        private TextWriterTraceListener m_listenerLog; ///< Bitacora de eventos del sistema
        private System.IO.FileStream m_traceLog; ///< Archivo enlazado con la bitacora de eventos del sistema
        #endregion

        public FrmMain()
        {
            #region Initialize
            InitializeComponent();
            new ToolTip().SetToolTip(pictureBox1, "Finalizar Programa");
            new ToolTip().SetToolTip(pictureBox2, "Configurar Parametros");
            new ToolTip().SetToolTip(this.lnkAgregarArchivo, "Agregar Archivo");
            new ToolTip().SetToolTip(this.lnkAgregarTicket, "Agregar  Nro. de Ticket");
            new ToolTip().SetToolTip(this.lnkBorrarEstadoConsulta, "Borrar Mensajes de Consulta");
            new ToolTip().SetToolTip(this.lnkBorrarEstadoEnvio, "Borrar Mensajes de Envio");
            new ToolTip().SetToolTip(this.lnkDir1, "Seleccionar Archivo");
            new ToolTip().SetToolTip(this.lnkDir2, "Seleccionar Archivo");
            new ToolTip().SetToolTip(this.lnkGrabarConfiguracion, "Grabar Configuración");
            new ToolTip().SetToolTip(this.lnkEliminarArchivo, "Eliminar archivo para enviar");
            new ToolTip().SetToolTip(this.lnkEliminarTicket, "Eliminar ticket para consultar");
            new ToolTip().SetToolTip(this.pctLimpiarLista, "Limpiar Lista de Consultas");
            m_dtEnvios = Logic.Core.CreateModelSend();
            m_dtConsultas = Logic.Core.CreateModelQuery();
            m_pathApp = Application.StartupPath;
            Logic.Core.VerifieDirectories(m_pathApp);
            CreateListener();
            this.txtEventoConsulta.Text = "";
            this.txtEventoEnvio.Text = "";
            Logic.Core.TraceLogApp("++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++");
            Logic.Core.TraceLogApp("Ruta de Ejecucion: " + m_pathApp);
            progressBar1.Minimum = 1;
            progressBar1.Value = 1;
            progressBar1.Step = 1;
            progressBar2.Minimum = 1;
            progressBar2.Value = 1;
            progressBar2.Step = 1;
            #endregion
        }

        #region Events 
        private void FrmMain_Load(object sender, EventArgs e)
        {
            LoadValues();
        }
        private void EvtClick1(object sender, EventArgs e)
        {
            CloseListener();
            Application.Exit();
        }

        private void EvtClick2(object sender, EventArgs e)
        {
            FrmConfigure oForm = new FrmConfigure();
            oForm.ShowDialog();
            if (oForm.Change)
            {
                LoadValues();
            }
        }

        private void EvtSelectFile(object sender, EventArgs e)
        {
            System.Windows.Forms.OpenFileDialog openfile = new OpenFileDialog();
            openfile.Filter = String.Format("Archivos Comprimidos (*.zip)|*.zip|Archivos Textos (*.txt)|*.txt");
            openfile.ShowDialog();
            this.txtArchivoEnvio.Text = openfile.FileName;
        }

        private void EvtAddFile(object sender, EventArgs e)
        {
            //Validando datos de entrada
            Boolean isTxt = false;
            if (0 == this.txtArchivoEnvio.Text.Trim().Length)
            {
                MessageBox.Show("El nombre del archivo no puede estar en blanco!", "Error de Validación");
                return;
            }

            if (File.Exists(this.txtArchivoEnvio.Text))
            {
                if (Logic.Core.EXTENSION_TXT == Path.GetExtension(this.txtArchivoEnvio.Text).ToLower())
                {
                    isTxt = true;
                }
            }
            else
            {
                MessageBox.Show("El archivo no existe!", "Error de Validación");
                return;
            }

            if (false == isTxt)
            {
                if (this.txtNroTransaccion.Text.Length < 4)
                {
                    MessageBox.Show("Número de Transacción invalido!", "Error de Validación");
                    return;
                }
                m_dtEnvios.Rows.Add(Logic.Core.NameUnique(), this.txtNroTransaccion.Text, this.txtArchivoEnvio.Text, "Pendiente");
            }
            else
            {
                int error = Logic.Core.AddSendFromFile("ENVIO", ref m_dtEnvios, this.txtArchivoEnvio.Text,"Pendiente");
                if (error > 0)
                {
                    MessageBox.Show("Se encontraron algunos errores en el archivo de Envio, algunos numeros de transaccion han sido descartados, revise el log para mas detalle.","Aviso");
                }
            }
            this.grdEnvios.DataSource = m_dtEnvios;
            this.grdEnvios.Refresh();
            this.txtNroTransaccion.Text = "";
            this.txtArchivoEnvio.Text = "";
            this.txtNroTransaccion.Focus();
        }

        private void EvtCleanMessage(object sender, EventArgs e)
        {
            this.txtEventoEnvio.Text = "";
        }

        /// <summary>
        /// Eventos para iniciar el envio de los archivos zip al servicio de SEIDA - SUNAT
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void cmdEnviar_Click(object sender, EventArgs e)
        {
            SendFiles();
        }

        private void EvtSaveConf(object sender, EventArgs e)
        {
            long intParsed = 0;
            if (this.txtRUC.Text.Length == 0)
            {
                MessageBox.Show("Número de RUC invalido!", "Error de Validación");
                return;
            }
            if (this.txtUsuario.Text.Length == 0)
            {
                MessageBox.Show("Usuario es invalido!", "Error de Validación");
                return;
            }
            if (this.txtCodigoEnvio.Text.Length == 0)
            {
                MessageBox.Show("Codigo de Envio es invalido!", "Error de Validación");
                return;
            }
            if (long.TryParse(this.txtRUC.Text.Trim(), out intParsed))
            {
                //Guardando los valores en el archivo de configuracion
                Configuration config = ConfigurationManager.OpenExeConfiguration(Application.ExecutablePath);
                config.AppSettings.Settings.Remove("NumeroRUC");
                config.AppSettings.Settings.Add("NumeroRUC", this.txtRUC.Text);
                config.AppSettings.Settings.Remove("UsuarioSOL");
                config.AppSettings.Settings.Add("UsuarioSOL", this.txtUsuario.Text);
                config.AppSettings.Settings.Remove("CodigoEnvio");
                config.AppSettings.Settings.Add("CodigoEnvio", this.txtCodigoEnvio.Text);
                config.Save(ConfigurationSaveMode.Modified);
                MessageBox.Show("Se ha guardado los parametros de configuracion.", "Guardado");
            }
            else
            {
                MessageBox.Show("Error, el número de RUC es incorrecto!", "Aviso");
            }
        }

        private void EvtAddTicket(object sender, EventArgs e)
        {
            if (this.txtNroTicket.Text.Trim().Length < 4)
            {
                MessageBox.Show("Número de Ticket invalido!", "Error de Validación");
                return;
            }
            long intParsed=0;
            if (long.TryParse(this.txtNroTicket.Text.Trim(), out intParsed))
            {
                m_dtConsultas.Rows.Add(Logic.Core.NameUnique(), this.picAno.Text, this.txtNroTicket.Text, "");
                this.grdConsultas.DataSource = m_dtConsultas;
                this.grdConsultas.Refresh();
                this.txtNroTicket.Text = "";
                this.txtNroTicket.Focus();
            }
            else
            {
                MessageBox.Show("Error, el formato del número de ticket es incorrecto!", "Aviso");
            }
        }

        private void EvtDeleteTicket(object sender, EventArgs e)
        {
            if (m_dtConsultas.Rows.Count == 0)
            {
                MessageBox.Show("No hay nada por eliminar.", "¿Eliminar?");
                return;
            }
            Int32 selectedRowCount = this.grdConsultas.Rows.GetRowCount(DataGridViewElementStates.Selected);
            if (selectedRowCount > 0)
            {
                int index = Logic.Core.GetIndexRowFromDataTable(m_dtConsultas, this.grdConsultas.SelectedRows[0].Cells[0].Value.ToString());
                if (index != -1)
                {
                    m_dtConsultas.Rows[index].Delete();
                    this.grdConsultas.DataSource = m_dtConsultas;
                    this.grdConsultas.Refresh();
                }
            }
        }

        private void EvtSelectTickets(object sender, EventArgs e)
        {
            System.Windows.Forms.OpenFileDialog openfile = new OpenFileDialog();
            openfile.Filter = String.Format("Archivos Textos (*.txt)|*.txt");
            openfile.ShowDialog();
            if (openfile.FileName.Trim() != "")
            {
                int error = Logic.Core.AddSendFromFile("CONSULTA", ref m_dtConsultas, openfile.FileName, "");
                if (error > 0)
                {
                    MessageBox.Show("Se encontraron algunos errores en el archivo de tickets, algunos tickets han sido descartados, revise el log para mas detalle.","Aviso");
                }
                this.grdConsultas.DataSource = m_dtConsultas;
                this.grdConsultas.Refresh();
            }
        }

        private void EvtCleanMessage2(object sender, EventArgs e)
        {
            this.txtEventoConsulta.Text = "";
        }

        private void cmdConsultar_Click(object sender, EventArgs e)
        {
            SendQuerys();
        }

        private void EvtDeleteFile(object sender, EventArgs e)
        {
            if (m_dtEnvios.Rows.Count == 0)
            {
                MessageBox.Show("No hay nada por eliminar.", "¿Eliminar?");
                return;
            }
            Int32 selectedRowCount = this.grdEnvios.Rows.GetRowCount(DataGridViewElementStates.Selected);
            if (selectedRowCount > 0)
            {
                int index = Logic.Core.GetIndexRowFromDataTable(m_dtEnvios, this.grdEnvios.SelectedRows[0].Cells[0].Value.ToString());
                if (index != -1)
                {
                    m_dtEnvios.Rows[index].Delete();
                    this.grdEnvios.DataSource = m_dtEnvios;
                    this.grdEnvios.Refresh();
                }
            }
        }

        private void pctLimpiarLista_Click(object sender, EventArgs e)
        {
            m_dtConsultas.Rows.Clear();
            this.grdConsultas.DataSource = m_dtConsultas;
            this.grdConsultas.Refresh();
        }
        private void EvtLeave(object sender, EventArgs e)
        {
            this.txtUsuario.Text = this.txtUsuario.Text.ToUpper();
        }
        #endregion

        #region Private Methods

        /// <summary>
        /// Realizar las consultas
        /// </summary>
        private void SendQuerys()
        {
            if (ValidatePreEnvio(2))
            {
                this.txtEventoConsulta.AppendText("  " + Environment.NewLine);
                this.txtEventoConsulta.AppendText("**Iniciando invocaciones:" + DateTime.Now.ToLongTimeString() + Environment.NewLine);
                Application.DoEvents();
                this.cmdConsultar.Enabled = false;
                this.lblEstadoConsulta.Visible = true;
                this.lblEstadoConsulta.ForeColor = System.Drawing.Color.Red;

                #region Ejecucion de hilos de consulta
                ArrayList objectsThread = new ArrayList();
                progressBar2.Maximum = (m_dtConsultas.Rows.Count * 2);
                progressBar2.Value = 1;
                progressBar2.Visible = true;
                for (int i = 0; i < m_dtConsultas.Rows.Count; i++)
                {
                    try
                    {
                        progressBar2.PerformStep();
                        Application.DoEvents();
                        Thread.Sleep(150);

                        Logic.ThreadProxy otp = new Logic.ThreadProxy("Task-" + i.ToString(), 2);
                        otp.Correlativo = (i+1);
                        otp.Url = m_url;
                        otp.Usuario = Logic.Core.CombineUserData(this.txtRUC.Text, this.txtUsuario.Text);
                        otp.Clave = this.txtClave.Text;
                        otp.IdProceso = m_dtConsultas.Rows[i][0].ToString();
                        otp.AnnoConsulta = m_dtConsultas.Rows[i][1].ToString();
                        otp.TicketConsulta = m_dtConsultas.Rows[i][2].ToString();
                        otp.DirectorioSave = m_pathApp + "\\" + Logic.Core.DIRECTORIO_RESPUESTA;
                        otp.TxtMessage = this.txtEventoConsulta;
                        this.txtEventoConsulta.AppendText("Consulta Hilo(" + otp.Correlativo.ToString() + ") | Año:" + otp.AnnoConsulta + " | Ticket: " + otp.TicketConsulta + Environment.NewLine);
                        Logic.Core.TraceLogApp("Consulta Hilo(" + otp.Correlativo.ToString() + ") | Año:" + otp.AnnoConsulta + " | Ticket: " + otp.TicketConsulta);
                        Application.DoEvents();
                        objectsThread.Add(otp);
                        otp.Run();
                        Application.DoEvents();
                    }
                    catch (Exception ex)
                    {
                        this.txtEventoConsulta.AppendText("Error: " + ex.Message.ToString() + Environment.NewLine);
                        Logic.Core.TraceLogApp("Error: " + ex.Message.ToString());
                    }
                }
                this.txtEventoEnvio.AppendText("Finalizando invocaciones: " + DateTime.Now.ToLongTimeString() + Environment.NewLine);
                this.txtEventoEnvio.AppendText("  " + Environment.NewLine);
                this.txtEventoEnvio.AppendText("Esperando finalizacion de hilos ..." + Environment.NewLine);
                this.txtEventoEnvio.AppendText("  " + Environment.NewLine);

                for (int j = 0; j < objectsThread.Count; j++)
                {
                    Logic.ThreadProxy otp = (Logic.ThreadProxy)objectsThread[j];
                    ///<Trace.WriteLine("Thread Nombre: " + otp.Namethread);
                    otp.Join();
                    progressBar2.PerformStep();
                    Application.DoEvents();
                }

                for (int j = 0; j < objectsThread.Count; j++)
                {
                    Logic.ThreadProxy otp = (Logic.ThreadProxy)objectsThread[j];
                    ///<Trace.WriteLine("Thread NombreRespuesta: " + otp.NombreRespuesta);
                    for (int k = 0; k < m_dtConsultas.Rows.Count; k++)
                    {
                        if (m_dtConsultas.Rows[k][0].ToString() == otp.IdProceso)
                        {
                            m_dtConsultas.Rows[k]["ArchivoRecibido"] = otp.NombreRespuesta;
                            break;
                        }
                    }
                }
                objectsThread.Clear();
                objectsThread = null;
                #endregion

                this.txtEventoConsulta.AppendText("Finalizando invocaciones: " + DateTime.Now.ToLongTimeString() + Environment.NewLine);
                this.txtEventoConsulta.AppendText("  " + Environment.NewLine);
                this.cmdConsultar.Enabled = true;
                this.lblEstadoConsulta.Visible = false;
                this.lblEstadoConsulta.ForeColor = System.Drawing.Color.Black;
                this.grdConsultas.DataSource = m_dtConsultas;
                this.grdConsultas.Refresh();
                Thread.Sleep(500);
                this.progressBar2.Visible = false;
                this.txtEventoConsulta.AppendText("Proceso terminado..." + Environment.NewLine);

                m_traceLog.Seek(0, SeekOrigin.End);
                m_listenerLog.Flush(); ///< Actualizando la bitacora de eventos
                m_traceLog.Flush();
            }
        }

        /// <summary>
        /// Envia los archivos ZIP
        /// </summary>
        private void SendFiles()
        {
            if (ValidatePreEnvio(1))
            {
                this.txtEventoEnvio.AppendText("  " + Environment.NewLine);
                this.txtEventoEnvio.AppendText("**INICIANDO invocaciones:" + DateTime.Now.ToLongTimeString() + Environment.NewLine);
                Application.DoEvents();
                this.cmdEnviar.Enabled = false;
                this.lblEstadoEnvio.Visible = true;
                this.lblEstadoEnvio.ForeColor = System.Drawing.Color.Red;

                #region Ejecucion de hilos de envio
                ArrayList objectsThread = new ArrayList();
                progressBar1.Maximum = (m_dtEnvios.Rows.Count * 2);
                progressBar1.Value = 1;
                progressBar1.Visible = true;
                Application.DoEvents();
                for (int i = 0; i < m_dtEnvios.Rows.Count; i++)
                {
                    try
                    {
                        progressBar1.PerformStep();
                        Application.DoEvents();
                        Thread.Sleep(500);

                        Logic.ThreadProxy otp = new Logic.ThreadProxy("Task-" + i.ToString(), 1);
                        otp.Correlativo = (i+1);
                        otp.Url = m_url;
                        otp.Usuario = Logic.Core.CombineUserData(this.txtRUC.Text, this.txtUsuario.Text);
                        otp.Clave = this.txtClave.Text;
                        otp.IdProceso = m_dtEnvios.Rows[i][0].ToString();
                        otp.NumeroTransaccion = m_dtEnvios.Rows[i][1].ToString();
                        otp.NombreArchivo = m_dtEnvios.Rows[i][2].ToString();
                        otp.DirectorioSave = m_pathApp + "\\" + Logic.Core.DIRECTORIO_ACUSE;
                        otp.TxtMessage = this.txtEventoEnvio;
                        this.txtEventoEnvio.AppendText("Enviando Hilo(" + otp.Correlativo.ToString() + ") | Nro.Transaccion:" + otp.NumeroTransaccion + " | Archivo: " + otp.NombreArchivo + Environment.NewLine);
                        Logic.Core.TraceLogApp("Enviando Hilo(" + otp.Correlativo.ToString() + ") | Nro.Transaccion:" + otp.NumeroTransaccion + " | Archivo: " + otp.NombreArchivo);
                        objectsThread.Add(otp);
                        Application.DoEvents();
                        otp.Run();
                    }
                    catch (Exception ex)
                    {
                        this.txtEventoEnvio.AppendText("Error: " + ex.Message.ToString() + Environment.NewLine);
                        Logic.Core.TraceLogApp("Error: " + ex.Message.ToString());
                    }
                }
                this.txtEventoEnvio.AppendText("Finalizando invocaciones: " + DateTime.Now.ToLongTimeString() + Environment.NewLine);
                this.txtEventoEnvio.AppendText("  " + Environment.NewLine);
                this.txtEventoEnvio.AppendText("Esperando finalizacion de hilos ..." + Environment.NewLine);
                this.txtEventoEnvio.AppendText("  " + Environment.NewLine);

                for (int j = 0; j < objectsThread.Count; j++)
                {
                    Logic.ThreadProxy otp = (Logic.ThreadProxy)objectsThread[j];
                    ///<Trace.WriteLine("Thread Nombre: " + otp.Namethread);
                    progressBar1.PerformStep();
                    Application.DoEvents();
                    otp.Join();
                }
                objectsThread.Clear();
                objectsThread = null;
                #endregion

                Thread.Sleep(500);
                this.progressBar1.Visible = false;
                this.cmdEnviar.Enabled = true;
                this.lblEstadoEnvio.Visible = false;
                this.lblEstadoEnvio.ForeColor = System.Drawing.Color.Black;
                m_dtEnvios.Rows.Clear();
                this.grdEnvios.DataSource = m_dtEnvios;
                this.grdEnvios.Refresh();
                this.txtEventoEnvio.AppendText("Proceso terminado..." + Environment.NewLine);
                this.txtEventoEnvio.AppendText("  " + Environment.NewLine);

                m_traceLog.Seek(0, SeekOrigin.End);
                m_listenerLog.Flush(); ///< Actualizando la bitacora de eventos
                m_traceLog.Flush();
            }
        }

        /// <summary>
        /// Leyendo los valores del archivo de configuracion en los controles y obteniendo la direccion IP y MAC de la computadora
        /// </summary>
        private void LoadValues()
        {
            bool change = false;
            Configuration config = ConfigurationManager.OpenExeConfiguration(Application.ExecutablePath);
            this.txtRUC.Text = config.AppSettings.Settings["NumeroRUC"].Value;
            this.txtUsuario.Text = config.AppSettings.Settings["UsuarioSOL"].Value;
            this.txtCodigoEnvio.Text = config.AppSettings.Settings["CodigoEnvio"].Value;
            m_url = config.AppSettings.Settings["UrlServicioWeb"].Value;
            this.lblUrlCurrent.Text = m_url;
            this.lblDireccionIP.Text = Logic.Core.GetIP();
            this.lblDireccionMAC.Text = Logic.Core.GetMACAddress();
            Logic.Core.TraceLogApp("NombrePC: " + Dns.GetHostName());
            Logic.Core.TraceLogApp("IP      : " + this.lblDireccionIP.Text);
            Logic.Core.TraceLogApp("MAC     : " + this.lblDireccionMAC.Text);

            string dir = "" + config.AppSettings.Settings["DirectorioDocumento"].Value;
            if (dir.Length == 0)
            {
                dir = m_pathApp + "\\" + Logic.Core.DIRECTORIO_DOCUMENTO;
                config.AppSettings.Settings.Remove("DirectorioDocumento");
                config.AppSettings.Settings.Add("DirectorioDocumento", dir);
                change = true;
            }
            dir = "" + config.AppSettings.Settings["DirectorioAcuse"].Value;
            if (dir.Length == 0)
            {
                dir = m_pathApp + "\\" + Logic.Core.DIRECTORIO_ACUSE;
                config.AppSettings.Settings.Remove("DirectorioAcuse");
                config.AppSettings.Settings.Add("DirectorioAcuse", dir);
                change = true;
            }
            dir = "" + config.AppSettings.Settings["DirectorioRespuesta"].Value;
            if (dir.Length == 0)
            {
                dir = m_pathApp + "\\" + Logic.Core.DIRECTORIO_RESPUESTA;
                config.AppSettings.Settings.Remove("DirectorioRespuesta");
                config.AppSettings.Settings.Add("DirectorioRespuesta", dir);
                change = true;
            }
            if (true == change)
            {
                config.Save(ConfigurationSaveMode.Modified);
            }
        }

        /// <summary>
        /// Validando los datos de entradas para realizar el envio de informacion
        /// </summary>
        /// <returns>Retorna falso si existe algun error en los datos, en caso contrario verdadero</returns>
        private Boolean ValidatePreEnvio(int typeValidate)
        {
            if (this.txtRUC.Text.Length == 0)
            {
                MessageBox.Show("Número de RUC invalido!", "Error de Validación");
                return false;
            }
            if (this.txtUsuario.Text.Length == 0)
            {
                MessageBox.Show("Usuario es invalido!", "Error de Validación");
                return false;
            }
            if (this.txtCodigoEnvio.Text.Length == 0)
            {
                MessageBox.Show("Codigo de Envio es invalido!", "Error de Validación");
                return false;
            }
            if (this.txtClave.Text.Length == 0)
            {
                MessageBox.Show("Clave SOL es invalido!", "Error de Validación");
                return false;
            }
            if (this.lblDireccionIP.Text.Length == 0)
            {
                MessageBox.Show("Error al obtener la direccion IP, no se podrá enviar y/o consultar.", "Error de Validación");
                return false;
            }
            if (this.lblDireccionMAC.Text.Length == 0)
            {
                MessageBox.Show("Error al obtener la direccion MAC, no se podrá enviar y/o consultar.", "Error de Validación");
                return false;
            }
            if (1 == typeValidate)
            {
                if (m_dtEnvios.Rows.Count == 0)
                {
                    MessageBox.Show("No hay archivos por enviar.", "¿Enviar?");
                    return false;
                }
            }
            else
            {
                if (m_dtConsultas.Rows.Count == 0)
                {
                    MessageBox.Show("No hay ticket por consultar.", "¿Enviar?");
                    return false;
                }
            }
            return true;
        }

        #endregion

        #region Listener Create and Close
        private void CreateListener()
        {
            if (File.Exists(m_pathApp + "\\" + Logic.Core.DIRECTORIO_LOGGER + "\\" + Logic.Core.TRACE_LOG_FILENAME))
                m_traceLog = new System.IO.FileStream(m_pathApp + "\\" + Logic.Core.DIRECTORIO_LOGGER + "\\" + Logic.Core.TRACE_LOG_FILENAME, System.IO.FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite);
            else
                m_traceLog = new System.IO.FileStream(m_pathApp + "\\" + Logic.Core.DIRECTORIO_LOGGER + "\\" + Logic.Core.TRACE_LOG_FILENAME, System.IO.FileMode.OpenOrCreate, FileAccess.ReadWrite, FileShare.ReadWrite);

            m_traceLog.Seek(0, SeekOrigin.End);
            m_listenerLog = new TextWriterTraceListener(m_traceLog);
            Trace.Listeners.Add(m_listenerLog);
        }
        private void CloseListener()
        {
            m_traceLog.Seek(0, SeekOrigin.End);
            m_listenerLog.Flush();
            m_traceLog.Flush();
            m_listenerLog.Close();
            m_listenerLog.Dispose();
            m_traceLog.Close();
            m_traceLog.Dispose();
            m_traceLog = null;
            m_listenerLog = null;
        }
        #endregion

    }
}

